unit Zitat;

{$mode objfpc}{$H+}

interface

uses
   Classes, SysUtils, ZitateUTF8, ZitateMsg;

{ TZitat }

type
   TZitat = class(TObject)
      constructor Create;
      constructor Create(zeile: string);
      constructor Create(cID, cKategorie, cNummer: integer; cSpruch, cLetzterZugriff: string);
      constructor Create(cKategorie, cNummer: integer; cSpruch: string);
      procedure Clear;
   private
      zID: integer;
      zKategorie: integer;
      zNummer: integer;
      zSpruch: string;
      zLetzterZugriff: string;
      zFehler: boolean;
      function getSpruch: string;
      procedure setSpruch (value: string);
      function getLetzterZugriff: string;
      procedure setLetzterZugriff (value: string);
   public
      function getGesamt: string;
      property ID: integer read zID write zID;
      property Kategorie: integer read zKategorie write zKategorie;
      property Nummer: integer read zNummer write zNummer;
      property Spruch: string read getSpruch write setSpruch;
      property letzterZugriff: string read getLetzterZugriff write setLetzterZugriff;
      property Fehler: boolean read zFehler write zFehler;
   end;

implementation

function aktuellesDatum: string;
begin
   Result:=FormatDateTime('dd.mm.yyyy', now);
end;

constructor TZitat.Create;
begin
   ID:= 0;
   Kategorie:= 0;
   Nummer:= 0;
   Spruch:= '';
   LetzterZugriff:= aktuellesDatum;
   Fehler:= false;
end;

procedure TZitat.Clear;
begin
   ID:= 0;
   Kategorie:= 0;
   Nummer:= 0;
   Spruch:= '';
   LetzterZugriff:= aktuellesDatum;
   Fehler:= false;
end;

constructor TZitat.Create(cID, cKategorie, cNummer: integer; cSpruch, cLetzterZugriff: string);
begin
   ID:= cID;
   Kategorie:= cKategorie;
   Nummer:= cNummer;
   Spruch:= cSpruch ;
   LetzterZugriff:= cLetzterZugriff ;
   Fehler:=false;
end;

constructor TZitat.Create(cKategorie, cNummer: integer; cSpruch: string);
begin
   ID:= -1;
   Kategorie:= cKategorie;
   Nummer:= cNummer;
   Spruch:= cSpruch;
   LetzterZugriff:= aktuellesDatum;
   Fehler:= false;
end;

constructor TZitat.Create(zeile: string);
var liste: TStringList;
begin
   try
      liste:=TStringList.Create;
      liste.Clear;
      liste.StrictDelimiter:= true;
      liste.Delimiter:= ';';
      liste.DelimitedText:= zeile;
      ID:= StrToInt( liste[0] );
      Kategorie:= StrToInt( liste[1] );
      Nummer:= StrToInt( liste[2] );
      Spruch:= liste[3];
      LetzterZugriff:= liste[4];
      Fehler:=false;
   except
      on E : EConvertError do begin
             Fehler:=true;
             SystemMeldung ('Fehler beim Konvertieren: ' + zeile);
      end;
   end;
end;

function TZitat.getGesamt: string;
begin
   Result:= IntToStr( ID) + ';' +
            IntToStr( Kategorie ) + ';' +
            IntToStr( Nummer ) + ';' +
            myReplace( Spruch, slinebreak, chr(255) )  + ';' +
            LetzterZugriff;
end;

function TZitat.getSpruch: string;
begin
   Result:= zSpruch;
end;

procedure TZitat.setSpruch (value: string);
begin
   zSpruch:= value;
end;

function TZitat.getLetzterZugriff: string;
begin
   Result:= zLetzterZugriff;
end;

procedure TZitat.setLetzterZugriff (value: string);
begin
   zLetzterZugriff:= value;
end;

begin

end.
