unit ZitateDB;

{$mode objfpc}{$H+}

interface

uses
   Classes, SysUtils, SQLite3Conn, SQLDB, DB, Zitat, ZitateListe, ZitateMsg;

{ TZitateDB }

type
   TZitateDB = class(TObject)
      SQLite3Connection1: TSQLite3Connection;
      SQLQuery1:          TSQLQuery;
      SQLQuery2:          TSQLQuery;
      SQLQuery3:          TSQLQuery;
      SQLTransaction1:    TSQLTransaction;

      constructor Create;
      function dbConnectOK(dbName: string): boolean;
      function tabellenAnlegenOK: boolean;
      function testTabellenOK: boolean;

      function getZitateView: TZitateListe;
      function getZitateListe: TZitateListe;
      function getZitateListe(startID, anzahl: integer): TZitateListe;
      function getZitateSet: TSQLQuery;
      function getZitateSetUpdate: TSQLQuery;
      procedure SpeichernSatz(Sender: TObject);
      procedure AddSatz(Sender: TObject);
      procedure AbbruchSatz(Sender: TObject);
      procedure Speichern(Sender: TObject);
      function getZitateSetSortiert(nr: integer; ab: boolean): TSQLQuery;

      procedure insertFast(iliste: TZitateListe);
      function insertZitat(zitat: TZitat): integer;
      procedure updateZitat(ID: integer; zitat: TZitat);
      procedure deleteZitat(ID: integer);
      procedure zapZitate;
      procedure zapZitateKategorie(minQuelle, maxQuelle: integer);

      function isZitatVorhanden(ID: integer): boolean;
      function isVorhanden(minQuelle, maxQuelle: integer): boolean;

      function getZitateAnzahl: integer;
      function getAnzahl(minQuelle, maxQuelle: integer): integer ;
      function getAnzahl(minQuelle, maxQuelle, kategorie, nummer: integer;
                         suche: boolean; stichwort: string): integer;

      function getKategorie(ID: integer): integer;
      function getNummer(ID: integer): integer;
      function getSpruch(ID: integer): string;

      function getZitat(ID: integer): TZitat;
      function getZitat(minQuelle, maxQuelle, kategorie, nummer: integer; suche: boolean;
                        stichwort: string; richtung, zufall: boolean;  vorgaenger: integer): TZitat;

      function getFreieKategorie: integer;
      function getFreieNummer(kategorie: integer): integer;

      function getLegalKategorie(wahlKategorie: integer): integer;
      function getLegalNummer(wahlNummer: integer): integer;

      function getMinID: integer;
      function getMaxID: integer;
      //Da kategorie + nummer + stichwort auch zusammen nicht eindeutig sind,
      //kann nicht direkt ID gesucht werden, nur min/max
      function getMinID(kategorie, nummer: integer): integer;
      function getMaxID(kategorie, nummer: integer): integer;
      function getMinID(kategorie, nummer: integer; stichwort: string): integer;
      function getMaxID(kategorie, nummer: integer; stichwort: string): integer;

      function getMinKategorie: integer;
      function getMaxKategorie: integer ;
      function getMinKategorie(minQuelle, maxQuelle: integer): integer;
      function getMaxKategorie(minQuelle, maxQuelle: integer): integer;

      function getMinNummer: integer;
      function getMaxNummer: integer;
      function getMinNummer(minQuelle, maxQuelle: integer): integer;
      function getMaxNummer(minQuelle, maxQuelle: integer): integer;
   private
      procedure setStichwort(value: string);
      function getStichwort: string;
   private
      dlPos: integer;
      dvonKategorie: integer;
      dbisKategorie: integer;
      dvonNummer: integer;
      dbisNummer: integer;
      dbStichwort: boolean;
      dsStichwort: string;
      sql: string;
      cursoreditaktuell: boolean;
      dbAnlegen: boolean;
   public
      property lPos: integer read dlpos write dlpos;
      property vonKategorie: integer read dvonKategorie write dvonKategorie;
      property bisKategorie: integer read dbisKategorie write dbisKategorie;
      property vonNummer: integer read dvonNummer write dvonNummer;
      property bisNummer: integer read dbisNummer write dbisNummer;
      property bStichwort: boolean read dbStichwort write dbStichwort;
      property sStichwort: string read getStichwort write setStichwort;
      property anlegen_erlaubt: boolean read dbAnlegen write dbAnlegen;
   end;

var
   zitateDB1: TZitateDB;

implementation

constructor TZitateDB.Create;
begin
   lPos:= -1;
   vonKategorie:= 1;
   bisKategorie:= 999999;
   vonNummer:= 1;
   bisNummer:= 999999;
   bStichwort:= false;
   sStichwort:= '';
   cursoreditaktuell:= true;
   dbAnlegen:= true;
end;

procedure TZitateDB.setStichwort(value: string);
begin
   dsStichwort:= value;
end;

function TZitateDB.getStichwort: string;
begin
   Result:=dsStichwort;
end;

function TZitateDB.dbConnectOK(dbName: string): boolean;
var dbOK: boolean;
begin
   dbOK:= true;
   try
      SQLite3Connection1:=TSQLite3Connection.Create(nil);
      SQLite3Connection1.DatabaseName:=dbName;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler:' +  ' SQLite3Connection1.Create '
                   + slinebreak + E.ClassName + E.Message );
      dbOK:= false;
      end;
   end;

   if dbOK then try
      SQLTransaction1:=TSQLTransaction.Create(nil);
      SQLTransaction1.Database:=SQLite3Connection1;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler:' +  ' SQLTransaction1.Create '
                   + slinebreak + E.ClassName + E.Message );
      dbOK:= false;
      end;
   end;

   if dbOK then try
      SQLQuery1:=TSQLQuery.Create(nil);
      SQLQuery1.Transaction:=SQLTransaction1;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler:' +  ' SQLQuery1.Create '
                   + slinebreak + E.ClassName + E.Message );
      dbOK:= false;
      end;
   end;

   if dbOK then try
      SQLQuery2:=TSQLQuery.Create(nil);
      SQLQuery2.Transaction:=SQLTransaction1;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler:' +  ' SQLQuery2.Create '
                   + slinebreak + E.ClassName + E.Message );
      dbOK:= false;
      end;
   end;

   if dbOK then try
      SQLQuery3:=TSQLQuery.Create(nil);
      SQLQuery3.Transaction:=SQLTransaction1;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler:' +  ' SQLQuery3.Create '
                   + slinebreak + E.ClassName + E.Message );
      dbOK:= false;
      end;
   end;

   Result:= dbOK;
end;

function TZitateDB.tabellenAnlegenOK: boolean;
var dbOK: boolean;
begin
   if not anlegen_erlaubt then exit;
   dbOK:= true;
   try
      sql:= ' CREATE TABLE IF NOT EXISTS tbZitate ( '
          + ' ID INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, '
          + ' KATEGORIE INTEGER NOT NULL, '
          + ' NUMMER INTEGER NOT NULL, '
          + ' SPRUCH TEXT NOT NULL, '
          + ' LETZTERZUGRIFF DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP ) ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.ExecSQL;
      SQLTransaction1.Commit;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler:' +  ' Create Table tbZitate '
                   + slinebreak + E.ClassName + E.Message );
      dbOK:= false;
      end;
   end;

   if dbOK then try
      sql:= ' CREATE INDEX IF NOT EXISTS ixNummer ON tbZitate (NUMMER) ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.ExecSQL;
      SQLTransaction1.Commit;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler:' +  ' Create Index ixNummer '
                   + slinebreak + E.ClassName + E.Message );
      dbOK:= false;
      end;
   end;

   if dbOK then try
      sql:= ' CREATE INDEX IF NOT EXISTS ixKategorie ON tbZitate (KATEGORIE) ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.ExecSQL;
      SQLTransaction1.Commit;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler:' +  ' Create Index ixKategorie '
                   + slinebreak + E.ClassName + E.Message );
      dbOK:= false;
      end;
   end;

   if dbOK then try
      sql:= ' CREATE VIEW IF NOT EXISTS vwZitate AS SELECT '
          + ' ID, KATEGORIE, NUMMER, '
          + ' SUBSTR( SPRUCH, 1, 40 ) AS KURZSPRUCH, '
          + ' LETZTERZUGRIFF FROM tbZitate ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.ExecSQL;
      SQLTransaction1.Commit;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler:' +  ' Create View vwZitate '
                   + slinebreak + E.ClassName + E.Message );
      dbOK:= false;
      end;
   end;

   Result:= dbOK;
end;

function TZitateDB.testTabellenOK: boolean;
var vorhanden: boolean;
begin
   try
      vorhanden:=false;
      SQLQuery1.Close;
      sql:= ' SELECT COUNT (*) FROM vwZitate ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      vorhanden:=(Length( SQLQuery1.Fields[0].asString ) > 0);
   except
      vorhanden:=false;
   end;
   Result:=vorhanden;
end;

function TZitateDB.getZitateView: TZitateListe;
var zitateliste: TZitateListe;
    zitat: TZitat;
begin
   try
      zitateliste:= TZitateListe.Create;
      SQLQuery2.Close;
      sql:= ' SELECT ID, kategorie, nummer, kurzspruch, letzterzugriff FROM vwZitate ';
      SQLQuery2.SQL.text:= sql;
      SQLQuery2.Open;
      while not SQLQuery2.Eof do begin
         zitat:= TZitat.Create( SQLQuery2.Fields[0].AsInteger,
                                SQLQuery2.Fields[1].AsInteger,
                                SQLQuery2.Fields[2].AsInteger,
                                SQLQuery2.Fields[3].AsString,
                                SQLQuery2.Fields[4].AsString);
         zitateliste.add( zitat );
         SQLQuery2.Next;
      end;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Liste aller Zitate View'
                   + slinebreak + E.ClassName + E.Message );
      end;
   end;
   Result:= zitateliste;
end;

function TZitateDB.getZitateListe: TZitateListe;
var zitateliste: TZitateListe;
    zitat: TZitat;
begin
   try
      zitateliste:= TZitateListe.Create;
      SQLQuery2.Close;
      sql:= ' SELECT ID, kategorie, nummer, spruch, letzterzugriff FROM tbZitate ';
      SQLQuery2.SQL.text:= sql;
      SQLQuery2.Open;
      while not SQLQuery2.Eof do begin
         zitat:= TZitat.Create( SQLQuery2.Fields[0].AsInteger,
                                SQLQuery2.Fields[1].AsInteger,
                                SQLQuery2.Fields[2].AsInteger,
                                SQLQuery2.Fields[3].AsString,
                                SQLQuery2.Fields[4].AsString );
         zitateliste.add( zitat );
         SQLQuery2.Next;
      end;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Liste aller Zitate'
                  + slinebreak + E.ClassName + E.Message );
      end;
   end;
   Result:= zitateliste;
end;

function TZitateDB.getZitateListe(startID, anzahl: integer): TZitateListe;
var zitateliste: TZitateListe;
    zitat: TZitat;
begin
   try
      zitateliste:= TZitateListe.Create;
      SQLQuery2.Close;
      sql:= ' SELECT ID, kategorie, nummer, spruch, letzterzugriff FROM tbZitate '
          + ' WHERE ID >= ' + IntToStr( startID )
          + ' LIMIT 0, ' + IntToStr( anzahl );
      SQLQuery2.SQL.text:= sql;
      SQLQuery2.Open;
      while not SQLQuery2.Eof do begin
         zitat:= TZitat.Create( SQLQuery2.Fields[0].AsInteger,
                                SQLQuery2.Fields[1].AsInteger,
                                SQLQuery2.Fields[2].AsInteger,
                                SQLQuery2.Fields[3].AsString,
                                SQLQuery2.Fields[4].AsString );
         zitateliste.add( zitat );
         SQLQuery2.Next;
      end;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Liste aller Zitate (Anzahl)'
                   + slinebreak + E.ClassName + E.Message );
      end;
   end;
   Result:= zitateliste;
end;

function TZitateDB.getZitateSet: TSQLQuery;
begin
   try
      SQLQuery3.Close;
      sql:= ' SELECT ID, kategorie, nummer, '
          + ' kurzspruch AS spruch, letzterzugriff AS aktuell '
          + ' FROM vwZitate '
          + ' WHERE ID > 0 ';

      if bStichwort
         then sql:= sql + ' AND spruch LIKE ''%' + sStichwort + '%'' ';
      if vonKategorie > 0
         then sql:= sql + ' AND kategorie >= ' + IntToStr( vonKategorie ) + ' ';
      if bisKategorie > 0
         then sql:= sql + ' AND kategorie <= ' + IntToStr( bisKategorie ) + ' ';
      if vonNummer > 0
         then sql:= sql + ' AND nummer >= ' + IntToStr( vonNummer ) + ' ';
      if bisNummer > 0
         then sql:= sql + ' AND nummer <= ' + IntToStr( bisNummer ) + ' ';

      SQLQuery3.SQL.text:= sql;
      SQLQuery3.Open;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' DBGrid Set Zitate View'
                   + slinebreak + E.ClassName + E.Message );
     end;
   end;
   Result:= SQLQuery3;
end;

function TZitateDB.getZitateSetUpdate: TSQLQuery;
begin
   try
      SQLQuery3.Close;
      sql:= ' SELECT ID, kategorie, nummer, spruch, letzterzugriff '
          + ' FROM tbZitate '
          + ' WHERE ID > 0 ';

      if bStichwort
         then sql:= sql + ' AND spruch LIKE ''%' + sStichwort + '%'' ';
      if vonKategorie > 0
         then sql:= sql + ' AND kategorie >= ' + IntToStr( vonKategorie ) + ' ';
      if bisKategorie > 0
         then sql:= sql + ' AND kategorie <= ' + IntToStr( bisKategorie ) + ' ';
      if vonNummer > 0
         then sql:= sql + ' AND nummer >= ' + IntToStr( vonNummer ) + ' ';
      if bisNummer > 0
         then sql:= sql + ' AND nummer <= ' + IntToStr( bisNummer ) + ' ';

      SQLQuery3.SQL.text:= sql;
      SQLQuery3.Open;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' DBGrid Set Zitate View'
                   + slinebreak + E.ClassName + E.Message );
     end;
   end;
   Result:= SQLQuery3;
end;

procedure TZitateDB.SpeichernSatz(Sender: TObject);
begin
   if SQLQuery3.State in [dsEdit, dsInsert]
      then SQLQuery3.Cancel;
end;

procedure TZitateDB.AddSatz(Sender: TObject);
begin
   Speichern(Nil);
   SQLQuery3.Append;
end;

procedure TZitateDB.AbbruchSatz(Sender: TObject);
begin
   if SQLQuery3.State in [dsEdit, dsInsert]
      then SQLQuery3.Cancel;
end;

procedure TZitateDB.Speichern(Sender: TObject);
begin
  if SQLQuery3.State In [dsEdit, dsInsert]
     then SQLQuery3.Post;
end;

function TZitateDB.getZitateSetSortiert(nr: integer; ab: boolean): TSQLQuery;
begin
   try
      SQLQuery3.Close;
      sql:= ' SELECT ID, kategorie, nummer, '
          + ' kurzspruch AS spruch, letzterzugriff AS aktuell '
          + ' FROM vwZitate '
          + ' WHERE ID > 0 ';

      if bStichwort
         then sql:= sql + ' AND spruch LIKE ''%' + sStichwort + '%'' ';
      if vonKategorie > 0
         then sql:= sql + ' AND kategorie >= ' + IntToStr( vonKategorie ) + ' ';
      if bisKategorie > 0
         then sql:= sql + ' AND kategorie <= ' + IntToStr( bisKategorie ) + ' ';
      if vonNummer > 0
         then sql:= sql + ' AND nummer >= ' + IntToStr( vonNummer ) + ' ';
      if bisNummer > 0
         then sql:= sql + ' AND nummer <= ' + IntToStr( bisNummer ) + ' ';

      sql:= sql + ' ORDER BY ';
      case nr of
         1: sql:= sql + ' kategorie ';
         2: sql:= sql + ' nummer ';
         3: sql:= sql + ' kurzspruch ';
         4: sql:= sql + ' letzterzugriff ';
         else sql:= sql + ' ID ';
      end;
      if ab
         then sql:= sql + ' DESC '
         else sql:= sql + ' ASC ';

      SQLQuery3.SQL.text:= sql;
      SQLQuery3.Open;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' DBGrid Set Zitate View Sortiert ' +
                   ' Nr= ' + IntToStr(nr) + slinebreak + E.ClassName + E.Message );
     end;
   end;
   Result:= SQLQuery3;
end;

procedure TZitateDB.insertFast(iliste: TZitateListe);
var i: integer;
    zitat: TZitat;
begin
   try
      SQLQuery1.Close;
      sql:= ' INSERT INTO tbZitate '
          + ' (kategorie, nummer, spruch, letzterzugriff ) '
          + ' VALUES ( :kategorie, :nummer, :spruch, CURRENT_TIMESTAMP ) ';
      SQLQuery1.SQL.text:= sql;

      for i:=1 to iliste.Anzahl do begin
         zitat:= iliste.getZitat(i);
         SQLQuery1.paramByName( 'kategorie' ).AsInteger := zitat.Kategorie;
         SQLQuery1.paramByName( 'nummer' ).AsInteger:= zitat.Nummer;
         SQLQuery1.paramByName( 'spruch' ).AsString := zitat.Spruch;
         SQLQuery1.Prepare;
         SQLQuery1.ExecSQL;
      end;

      SQLTransaction1.Commit;
  except
  on E: EDatabaseError do begin
     SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Fast Insert '
                  + slinebreak + E.ClassName + E.Message );
     SQLTransaction1.Rollback;
     end;
  end;
end;

function TZitateDB.InsertZitat(zitat: TZitat): integer;
var wert: integer;
begin
   wert:= -1;
   try
      SQLQuery1.Close;
      sql:= ' INSERT INTO tbZitate '
          + ' (kategorie, nummer, spruch, letzterzugriff ) '
          + ' VALUES ( :kategorie, :nummer, :spruch, CURRENT_TIMESTAMP ) ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.paramByName( 'kategorie' ).AsInteger:= zitat.Kategorie;
      SQLQuery1.paramByName( 'nummer' ).AsInteger:= zitat.Nummer;
      SQLQuery1.paramByName( 'spruch' ).AsString := zitat.Spruch;
      SQLQuery1.ExecSQL;
      // Wichtig! In der laufenden Transaktion nach dem gerade eingefgten
      // Satz fahnden
      wert:= getMaxID;
      SQLTransaction1.Commit;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Insert Zitat'
                   + slinebreak + E.ClassName + E.Message );
      SQLTransaction1.Rollback;
      wert:= -1;
      end;
   end;
   Result:= wert;
end;

procedure TZitateDB.updateZitat(ID: integer; zitat: TZitat);
begin
   try
      SQLQuery1.Close;
      sql:= ' UPDATE tbZitate '
          + ' SET kategorie = :kategorie, nummer = :nummer, '
          + '     spruch = :spruch, letzterzugriff = CURRENT_TIMESTAMP '
          + ' WHERE ID = :ID ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.paramByName( 'ID' ).AsInteger:= ID;
      SQLQuery1.paramByName( 'kategorie' ).AsInteger:= zitat.Kategorie;
      SQLQuery1.paramByName( 'nummer' ).AsInteger:= zitat.Nummer;
      SQLQuery1.paramByName( 'spruch' ).AsString := zitat.Spruch;
      SQLQuery1.ExecSQL;
      SQLTransaction1.Commit;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Update Zitat'
                   + slinebreak + E.ClassName + E.Message );
      SQLTransaction1.Rollback;
      end;
   end;
end;

procedure TZitateDB.deleteZitat(ID: integer);
begin
   try
      SQLQuery1.Close;
      sql:= ' DELETE FROM tbZitate '
          + ' WHERE ID = ' + IntToStr( ID );
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.ExecSQL;
      SQLTransaction1.Commit;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Delete Zitat'
                   + slinebreak + E.ClassName + E.Message );
      SQLTransaction1.Rollback;
      end;
   end;
end;

procedure TZitateDB.zapZitate;
begin
   try
      SQLQuery1.Close;
      sql:= ' DELETE FROM tbZitate ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.ExecSQL;
      SQLTransaction1.Commit;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff ZAP '
                   + slinebreak + E.ClassName + E.Message );
      SQLTransaction1.Rollback;
      end;
   end;
end;

procedure TZitateDB.zapZitateKategorie(minQuelle, maxQuelle: integer);
begin
   try
      SQLQuery1.Close;
      sql:= ' DELETE FROM tbZitate '
          + ' WHERE kategorie BETWEEN ' + IntToStr( minQuelle )
          +                     ' AND ' + IntToStr( maxQuelle );
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.ExecSQL;
      SQLTransaction1.Commit;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff ZAP/Kategorien '
                   + slinebreak + E.ClassName + E.Message );
      SQLTransaction1.Rollback;
      end;
   end;
end;

function TZitateDB.isZitatVorhanden(ID: integer): boolean;
var wert: integer;
    vorhanden: boolean;
begin
   try
      wert:= 0;
      vorhanden:= false;
      SQLQuery1.Close;
         sql:= ' SELECT COUNT(*) FROM tbZitate '
             + ' WHERE ID = ' + IntToStr( ID );
         SQLQuery1.SQL.text:= sql;
         SQLQuery1.Open;
         if Length( SQLQuery1.Fields[0].asString ) > 0 then begin
            wert:= SQLQuery1.Fields[0].AsInteger;
            vorhanden:= (wert > 0);
         end;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff Exists '
                   + slinebreak + E.ClassName + E.Message );
      vorhanden:= false;
      end;
   end;
   Result:= vorhanden;
end;

function TZitateDB.isVorhanden(minQuelle, maxQuelle: integer): boolean;
var wert: integer;
    vorhanden: boolean;
begin
   try
      wert:= 0;
      vorhanden:= false;
      SQLQuery1.Close;
      sql:= ' SELECT CASE '
          + ' WHEN EXISTS ( '
          + '               SELECT *  '
          + '               FROM tbzitate '
          + '               WHERE kategorie BETWEEN ' + IntToStr( minQuelle )
          +                                   ' AND ' + IntToStr( maxQuelle )
          + '      ) THEN 1 ELSE 0 END ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0 then begin
         wert:= SQLQuery1.Fields[0].AsInteger;
         vorhanden:= (wert > 0);
      end;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff isVorhanden/Kategorien '
                 + slinebreak + E.ClassName + E.Message );
      vorhanden:= false;
      end;
   end;
   Result:= vorhanden;
end;

function TZitateDB.getZitateAnzahl: integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT COUNT(*) FROM tbZitate WHERE ID > 0 ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff Anzahl alle Zitate '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getAnzahl(minQuelle, maxQuelle: integer): integer ;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT COUNT(*) FROM tbZitate '
          + ' WHERE kategorie BETWEEN ' + IntToStr( minQuelle )
          +                     ' AND ' + IntToStr( maxQuelle );
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff Anzahl Zitate in den Kategorien '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getAnzahl(minQuelle, maxQuelle, kategorie, nummer: integer;
                             suche: boolean; stichwort: string): integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT COUNT(*) FROM tbZitate '
          + ' WHERE kategorie BETWEEN ' + IntToStr( minQuelle )
          +                     ' AND ' + IntToStr( maxQuelle );

      if suche
         then sql:= sql + ' AND spruch LIKE ''%' + stichwort + '%'' ';
      if ( kategorie > 0 ) and ( nummer <= 0 )
         then sql:= sql + ' AND kategorie = ' + IntToStr( kategorie ) + ' '
      else if ( nummer > 0 ) and ( kategorie <= 0 )
         then sql:= sql + ' AND nummer = ' + IntToStr( nummer ) + ' '
      else if ( nummer > 0 ) and ( kategorie > 0 )
         then sql:= sql + ' AND kategorie = ' + IntToStr( kategorie ) + ' '
                        + ' AND nummer = ' + IntToStr( nummer ) + ' ' ;

      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff Anzahl Zitate (komplex) '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getKategorie(ID: integer): integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT kategorie FROM tbZitate '
          + ' WHERE ID = ' + IntToStr( ID );
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff Kategorie zu ID'
                 + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getNummer(ID: integer): integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT nummer FROM tbZitate '
          + ' WHERE ID = ' + IntToStr( ID );
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff Nummer zu ID'
                 + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getSpruch(ID: integer): string;
var spruch: string;
begin
   try
      spruch:= '';
      SQLQuery1.Close;
      sql:= ' SELECT spruch FROM tbZitate '
          + ' WHERE ID = ' + IntToStr( ID );
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then spruch:= SQLQuery1.Fields[0].AsString;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff Spruch zu ID'
                 + slinebreak + E.ClassName + E.Message );
      spruch:= '';
      end;
   end;
   Result:= spruch;
end;

function TZitateDB.getZitat(ID: integer): TZitat;
var zitat: TZitat;
begin
   try
      zitat:= TZitat.Create;
      SQLQuery1.Close;
      sql:= ' SELECT ID, kategorie, nummer, spruch, letzterzugriff FROM tbZitate '
          + ' WHERE ID = ' + IntToStr( ID );
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then begin
           zitat.ID:=             SQLQuery1.Fields[0].AsInteger;
           zitat.Kategorie:=      SQLQuery1.Fields[1].AsInteger;
           zitat.Nummer:=         SQLQuery1.Fields[2].AsInteger;
           zitat.Spruch:=         SQLQuery1.Fields[3].AsString;
           zitat.letzterZugriff:= SQLQuery1.Fields[4].AsString;
      end;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff Zitat'
                   + slinebreak + E.ClassName + E.Message );
      end;
   end;
   Result:= zitat;
end;

function TZitateDB.getZitat(minQuelle, maxQuelle, kategorie, nummer: integer; suche: boolean;
                            stichwort: string; richtung, zufall: boolean;  vorgaenger: integer): TZitat;
var zitat: TZitat;
    anzahl: integer;
begin
   try
      zitat:= TZitat.Create;

      anzahl:= getAnzahl( minQuelle, maxQuelle, kategorie, nummer, suche, stichwort );
      if anzahl > 0 then begin

         lPos:= vorgaenger;
         if zufall
            then lPos:= Random(anzahl)
            else begin
               if richtung
                  then lPos:= vorgaenger + 1
                  else lPos:= vorgaenger - 1;
               if lPos >= anzahl
                  then lPos:= 0
                  else if lPos < 0
                          then lPos:= anzahl - 1;
         end;

         SQLQuery1.Close;

         //minQuelle, maxQuelle sind nicht sinnvoll Null
         //vonKategorie, bisKategorie, kategorie und nummer dagegen schon
         //Daher ist die Unterscheidung bei BETWEEN und <= und >= gewollt
         sql:= ' SELECT ID, kategorie, nummer, spruch, letzterzugriff '
             + ' FROM tbZitate '
             + ' WHERE kategorie BETWEEN ' + IntToStr( minQuelle )
             +                     ' AND ' + IntToStr( maxQuelle );

         if suche
            then sql:= sql + ' AND spruch LIKE ''%' + stichwort + '%'' ';
         if ( kategorie > 0 ) and ( nummer <= 0 )
            then sql:= sql + ' AND kategorie = ' + IntToStr( kategorie ) + ' '
         else if ( nummer > 0 ) and ( kategorie <= 0 )
            then sql:= sql + ' AND nummer = ' + IntToStr( nummer ) + ' '
         else if ( nummer > 0 ) and ( kategorie > 0 )
            then sql:= sql + ' AND kategorie = ' + IntToStr( kategorie ) + ' '
                           + ' AND nummer = ' + IntToStr( nummer ) + ' ' ;

         //lPos enthlt hier schon die neue Position
         //Limit m,1 bedeutet nur 1 Zeile
         //(deshalb kann der Zugriff auch ohne Key ID erfolgen)
         sql:= sql + ' LIMIT ' + IntToStr(lPos) + ' , 1 ';

         SQLQuery1.SQL.text:= sql;
         SQLQuery1.Open;
         if Length( SQLQuery1.Fields[0].asString ) > 0
            then begin
              zitat.ID:=             SQLQuery1.Fields[0].AsInteger;
              zitat.Kategorie:=      SQLQuery1.Fields[1].AsInteger;
              zitat.Nummer:=         SQLQuery1.Fields[2].AsInteger;
              zitat.Spruch:=         SQLQuery1.Fields[3].AsString;
              zitat.letzterZugriff:= SQLQuery1.Fields[4].AsString;
         end;
      end;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff Zitat komplex'
                   + slinebreak + E.ClassName + E.Message );
      end;
   end;
   Result:= zitat;
end;

function TZitateDB.getFreieKategorie: integer;
var wert: integer;
begin
   try
      wert:= 1;
      SQLQuery1.Close;
      sql:= ' SELECT MIN (kategorie + 1) '
          + ' FROM tbZitate '
          + ' WHERE NOT ( kategorie + 1 IN  '
          + '      ( SELECT kategorie '
          + '        FROM tbzitate )) ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
      //Sonderflle:
      //- Null wird leider als 0 zurckgegeben
      //- Kategorie 1 fehlt (egal wie gro die Lcke danach ist)
      if (wert = 0) or (getAnzahl( 1, 1 ) = 0)
         then wert:= 1;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff nchste freie Kategorie '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 1;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getFreieNummer(kategorie: integer): integer;
var wert: integer;
begin
   try
      wert:= 1;
      SQLQuery1.Close;
      sql:= ' SELECT MIN (nummer + 1) '
          + ' FROM tbZitate '
          + ' WHERE NOT ( nummer + 1 IN  '
          + '      ( SELECT nummer '
          + '        FROM tbzitate '
          + '        WHERE kategorie = ' + IntToStr( kategorie ) + ' )) ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
      //Sonderflle:
      //- Null wird leider als 0 zurckgegeben
      if wert = 0
         then wert:= 1;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff nchste freie Nummer '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 1;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getLegalKategorie(wahlKategorie: integer): integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MAX (kategorie) '
          + ' FROM tbZitate '
          + ' WHERE kategorie IN  '
          + '       ( SELECT kategorie '
          + '         FROM   tbzitate  '
          + '         WHERE  kategorie <= ' + IntToStr( wahlKategorie ) + ' ) ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff legal(kategorie) '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getLegalNummer(wahlNummer: integer): integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MAX (nummer) '
          + ' FROM tbZitate '
          + ' WHERE nummer IN  '
          + '       ( SELECT nummer '
          + '         FROM   tbzitate  '
          + '         WHERE  nummer <= ' + IntToStr( wahlNummer ) + ' ) ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff legal(nummer) '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getMinID: integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MIN (ID) FROM tbZitate ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff kleinste ID '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getMaxID: integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MAX (ID) FROM tbZitate ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff hchste ID '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getMinID(kategorie, nummer: integer): integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MIN (ID) FROM tbZitate '
         +  ' WHERE ID > 0 ';

      if ( kategorie > 0 ) and ( nummer <= 0 )
         then sql:= sql + ' AND kategorie = ' + IntToStr( kategorie ) + ' '
      else if ( nummer > 0 ) and ( kategorie <= 0 )
         then sql:= sql + ' AND nummer = ' + IntToStr( nummer ) + ' '
      else if ( nummer > 0 ) and ( kategorie > 0 )
         then sql:= sql + ' AND kategorie = ' + IntToStr( kategorie ) + ' '
                        + ' AND nummer = ' + IntToStr( nummer ) + ' ' ;

      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff kleinste ID (kategorie, nummer) '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getMaxID(kategorie, nummer: integer): integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MAX (ID) FROM tbZitate '
         +  ' WHERE ID > 0 ';

      if ( kategorie > 0 ) and ( nummer <= 0 )
         then sql:= sql + ' AND kategorie = ' + IntToStr( kategorie ) + ' '
      else if ( nummer > 0 ) and ( kategorie <= 0 )
         then sql:= sql + ' AND nummer = ' + IntToStr( nummer ) + ' '
      else if ( nummer > 0 ) and ( kategorie > 0 )
         then sql:= sql + ' AND kategorie = ' + IntToStr( kategorie ) + ' '
                        + ' AND nummer = ' + IntToStr( nummer ) + ' ' ;

      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff grte ID (kategorie, nummer) '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getMinID(kategorie, nummer: integer; stichwort: string): integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MIN (ID) FROM tbZitate '
         +  ' WHERE ID > 0 ';

      if stichwort <> ''
         then sql:= sql + ' AND spruch LIKE ''%' + stichwort + '%'' ';
      if ( kategorie > 0 ) and ( nummer <= 0 )
         then sql:= sql + ' AND kategorie = ' + IntToStr( kategorie ) + ' '
      else if ( nummer > 0 ) and ( kategorie <= 0 )
         then sql:= sql + ' AND nummer = ' + IntToStr( nummer ) + ' '
      else if ( nummer > 0 ) and ( kategorie > 0 )
         then sql:= sql + ' AND kategorie = ' + IntToStr( kategorie ) + ' '
                        + ' AND nummer = ' + IntToStr( nummer ) + ' ' ;

      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff kleinste ID (kategorie, nummer, stichwort) '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getMaxID(kategorie, nummer: integer; stichwort: string): integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MAX (ID) FROM tbZitate '
         +  ' WHERE ID > 0 ';

      if stichwort <> ''
         then sql:= sql + ' AND spruch LIKE ''%' + stichwort + '%'' ';
      if ( kategorie > 0 ) and ( nummer <= 0 )
         then sql:= sql + ' AND kategorie = ' + IntToStr( kategorie ) + ' '
      else if ( nummer > 0 ) and ( kategorie <= 0 )
         then sql:= sql + ' AND nummer = ' + IntToStr( nummer ) + ' '
      else if ( nummer > 0 ) and ( kategorie > 0 )
         then sql:= sql + ' AND kategorie = ' + IntToStr( kategorie ) + ' '
                        + ' AND nummer = ' + IntToStr( nummer ) + ' ' ;

      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff grte ID (kategorie, nummer, stichwort) '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getMinKategorie: integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MIN (kategorie) FROM tbZitate ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff kleinste Kategorie '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getMaxKategorie: integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MAX (kategorie) FROM tbZitate ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff hchste Kategorie '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getMinKategorie(minQuelle, maxQuelle: integer): integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MIN (kategorie) FROM tbZitate '
          + ' WHERE kategorie BETWEEN ' + IntToStr( minQuelle )
          +                     ' AND ' + IntToStr( maxQuelle );
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff kleinste Kategorie/Quellen '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getMaxKategorie(minQuelle, maxQuelle: integer): integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MAX (kategorie) FROM tbZitate '
          + ' WHERE kategorie BETWEEN ' + IntToStr( minQuelle )
          +                     ' AND ' + IntToStr( maxQuelle );
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff hchste Kategorie/Quellen '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getMinNummer: integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MIN (nummer) FROM tbZitate ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff kleinste Nummer '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getMaxNummer: integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MAX (nummer) FROM tbZitate ';
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff hchste Nummer '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getMinNummer(minQuelle, maxQuelle: integer): integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MIN (nummer) FROM tbZitate '
          + ' WHERE kategorie BETWEEN ' + IntToStr( minQuelle )
          +                     ' AND ' + IntToStr( maxQuelle );
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff kleinste Nummer/Quellen '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

function TZitateDB.getMaxNummer(minQuelle, maxQuelle: integer): integer;
var wert: integer;
begin
   try
      wert:= 0;
      SQLQuery1.Close;
      sql:= ' SELECT MAX (nummer) FROM tbZitate '
          + ' WHERE kategorie BETWEEN ' + IntToStr( minQuelle )
          +                     ' AND ' + IntToStr( maxQuelle );
      SQLQuery1.SQL.text:= sql;
      SQLQuery1.Open;
      if Length( SQLQuery1.Fields[0].asString ) > 0
         then wert:= SQLQuery1.Fields[0].AsInteger;
   except
   on E: EDatabaseError do begin
      SystemMeldung( ' Datenbank Fehler Tabelle tbZitate:' +  ' Zugriff hchste Nummer/Quellen '
                   + slinebreak + E.ClassName + E.Message );
      wert:= 0;
      end;
   end;
   Result:= wert;
end;

begin
   Randomize;
   zitateDB1:=TZitateDB.Create;
end.

