unit ZitateListe;

{$mode objfpc}{$H+}

interface

uses
   Classes, SysUtils, Fgl, Zitat, ZitateUTF8;

{ TZitateListe }

type
   TZitateGeneric = specialize TFPGObjectList<TZitat>;
   TZitateListe = class(TObject)
   public
      constructor Create;
      function getZitat(nr: integer): TZitat;
      procedure Clear;
      procedure add(izitat: TZitat);
      procedure zitateDatei(sDateiname: string);
   private
      function getAnzahl: integer;
      procedure zitateLesen;
      procedure auswerten(zeile: string);
      function konvert(zeile: string): string;
   private
      liste: TZitateGeneric;
      spruch: string;
      lkategorie: integer;
      lvorspann: integer;
      dateiname:string;
      datei: text;
   public
      property Kategorie: integer read lkategorie write lkategorie;
      property Vorspann: integer read lvorspann write lvorspann;
      property Anzahl: integer read getAnzahl;
   end;

implementation

constructor TZitateListe.Create;
begin
   Kategorie:= 0;
   Vorspann:= 0;
   spruch:= '';
   dateiname:= '';
   liste:= TZitateGeneric.Create( true );
end;

procedure TZitateListe.Clear;
begin
   liste.Clear;
end;

procedure TZitateListe.add(izitat: TZitat);
begin
   liste.add( izitat );
end;

function TZitateListe.getAnzahl: integer;
begin
   Result:= liste.Count;
end;

function TZitateListe.getZitat(nr: integer): TZitat;
begin
   Result:= liste.Items[ nr-1 ];
end;

procedure TZitateListe.zitateDatei(sDateiname: string);
begin
   dateiname:= Sdateiname;
   try
      if not FileExists( dateiname )
         then WriteUTF8Ln('Datei: ' + dateiname + ' existiert nicht')
         else zitateLesen;
   except
   on E: EInOutError do
      WriteUTF8Ln( 'Fehler beim ffnen ' + dateiname
                   + slinebreak + E.ClassName + E.Message );
   end;
end;

procedure TZitateListe.zitateLesen;
var i: integer;
    zeile: string;
begin
   liste.Clear;
   try
      assignFile( datei, dateiname );
      reset( datei );
      i:= 0;

      readln( datei, zeile );
      while not eof(datei) do begin
         inc( i );
         if i > vorspann
            then auswerten (zeile);
         readln( datei, zeile);
      end;

      if Length( spruch ) > 0
         then auswerten ( '###' );
   except
   on E: EInOutError do
      WriteUTF8Ln( 'Fehler beim Zitat-Erstellen ' + dateiname + E.ClassName + slinebreak + E.Message )
   end;

   try
      close( datei );
   except
   on E: EInOutError do
      WriteUTF8Ln( 'Fehler beim Close ' + dateiname + E.ClassName + slinebreak + E.Message );
   end;
end;

procedure TZitateListe.auswerten(zeile: string);
var zitat: TZitat;
begin
   if Length( zeile ) = 0
      then zeile:= ' ';

   if myContains( zeile, '###' ) then begin
      if Length( Trim( spruch ) ) > 0 then begin
         zitat:= TZitat.Create( kategorie, -1, spruch );
         liste.Add( zitat );
      end;
      spruch:='';  Exit;
   end;

   if myContains( zeile, '~~~' ) then begin
      kategorie:= kategorie + 1;
      Exit;
   end;

   if Length( spruch ) = 0
      then spruch:= konvert( zeile )
      else spruch:= spruch + slinebreak + konvert( zeile );
end;

function TZitateListe.konvert(zeile: string): string;
begin
   Result:= Ansi2UTF8( myReplace ( zeile, ';', ',' ) );
end;

begin

end.
